@@ -21,7 +21,7 @@ import logging
 from osc_lib.command import command
 from osc_lib import utils as oscutils
 
-from ironicclient.common.i18n import _
+from ironicclient.common.i18n import _, _LW
 from ironicclient.common import utils
 from ironicclient import exc
 from ironicclient.v1 import resource_fields as res_fields
@@ -322,6 +322,7 @@ class CreateBaremetalNode(command.ShowOne):
 
         node.pop('links', None)
         node.pop('ports', None)
+        node.pop('portgroups', None)
 
         node.setdefault('chassis_uuid', '')
 
@@ -713,6 +714,21 @@ class PowerBaremetalNode(command.Command):
             metavar='<node>',
             help="Name or UUID of the node."
         )
+        parser.add_argument(
+            '--soft',
+            dest='soft',
+            action='store_true',
+            default=False,
+            help=_("Request graceful power-off.")
+        )
+        parser.add_argument(
+            '--power-timeout',
+            metavar='<power-timeout>',
+            default=None,
+            type=int,
+            help=_("Timeout (in seconds, positive integer) to wait for the "
+                   "target power state before erroring out.")
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -721,7 +737,8 @@ class PowerBaremetalNode(command.Command):
         baremetal_client = self.app.client_manager.baremetal
 
         baremetal_client.node.set_power_state(
-            parsed_args.node, parsed_args.power_state)
+            parsed_args.node, parsed_args.power_state, parsed_args.soft,
+            timeout=parsed_args.power_timeout)
 
 
 class ProvideBaremetalNode(ProvisionStateBaremetalNode):
@@ -742,7 +759,23 @@ class RebootBaremetalNode(command.Command):
         parser.add_argument(
             'node',
             metavar='<node>',
-            help="Name or UUID of the node.")
+            help="Name or UUID of the node."
+        )
+        parser.add_argument(
+            '--soft',
+            dest='soft',
+            action='store_true',
+            default=False,
+            help=_("Request Graceful reboot.")
+        )
+        parser.add_argument(
+            '--power-timeout',
+            metavar='<power-timeout>',
+            default=None,
+            type=int,
+            help=_("Timeout (in seconds, positive integer) to wait for the "
+                   "target power state before erroring out.")
+        )
 
         return parser
 
@@ -752,7 +785,8 @@ class RebootBaremetalNode(command.Command):
         baremetal_client = self.app.client_manager.baremetal
 
         baremetal_client.node.set_power_state(
-            parsed_args.node, 'reboot')
+            parsed_args.node, 'reboot', parsed_args.soft,
+            timeout=parsed_args.power_timeout)
 
 
 class RebuildBaremetalNode(ProvisionStateBaremetalNode):
@@ -785,6 +819,11 @@ class SetBaremetalNode(command.Command):
             metavar="<name>",
             help="Set the name of the node",
         )
+        parser.add_argument(
+            "--chassis-uuid",
+            metavar="<chassis UUID>",
+            help="Set the chassis for the node",
+        )
         parser.add_argument(
             "--driver",
             metavar="<driver>",
@@ -864,6 +903,10 @@ class SetBaremetalNode(command.Command):
             name = ["name=%s" % parsed_args.name]
             properties.extend(utils.args_array_to_patch(
                 'add', name))
+        if parsed_args.chassis_uuid:
+            chassis_uuid = ["chassis_uuid=%s" % parsed_args.chassis_uuid]
+            properties.extend(utils.args_array_to_patch(
+                'add', chassis_uuid))
         if parsed_args.driver:
             driver = ["driver=%s" % parsed_args.driver]
             properties.extend(utils.args_array_to_patch(
@@ -893,6 +936,8 @@ class SetBaremetalNode(command.Command):
                         in parsed_args.instance_info]))
         if properties:
             baremetal_client.node.update(parsed_args.node, properties)
+        else:
+            self.log.warning(_LW("Please specify what to set."))
 
 
 class SetBaremetal(SetBaremetalNode):
@@ -951,6 +996,7 @@ class ShowBaremetalNode(command.ShowOne):
                 parsed_args.node, fields=fields)._info
         node.pop("links", None)
         node.pop("ports", None)
+        node.pop('portgroups', None)
 
         node.setdefault('chassis_uuid', '')
 
@@ -1038,6 +1084,12 @@ class UnsetBaremetalNode(command.Command):
             help='Instance information to unset on this baremetal node '
                  '(repeat option to unset multiple instance informations)',
         )
+        parser.add_argument(
+            "--chassis-uuid",
+            dest='chassis_uuid',
+            action='store_true',
+            help=_('Unset chassis UUID on this baremetal node'),
+        )
 
         return parser
 
@@ -1077,8 +1129,13 @@ class UnsetBaremetalNode(command.Command):
             properties.extend(utils.args_array_to_patch('remove',
                               ['instance_info/' + x for x
                                in parsed_args.instance_info]))
+        if parsed_args.chassis_uuid:
+            properties.extend(utils.args_array_to_patch('remove',
+                              ['chassis_uuid']))
         if properties:
             baremetal_client.node.update(parsed_args.node, properties)
+        else:
+            self.log.warning(_LW("Please specify what to unset."))
 
 
 class UnsetBaremetal(UnsetBaremetalNode):
@@ -1124,3 +1181,95 @@ class ValidateBaremetalNode(command.Lister):
         data = oscutils.sort_items(data, 'interface')
         return (field_labels,
                 (oscutils.get_dict_properties(s, fields) for s in data))
+
+
+class VifListBaremetalNode(command.Lister):
+    """Show attached VIFs for a node"""
+
+    log = logging.getLogger(__name__ + ".VifListBaremetalNode")
+
+    def get_parser(self, prog_name):
+        parser = super(VifListBaremetalNode, self).get_parser(prog_name)
+
+        parser.add_argument(
+            'node',
+            metavar='<node>',
+            help="Name or UUID of the node"
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        self.log.debug("take_action(%s)", parsed_args)
+
+        columns = res_fields.VIF_RESOURCE.fields
+        labels = res_fields.VIF_RESOURCE.labels
+
+        baremetal_client = self.app.client_manager.baremetal
+        data = baremetal_client.node.vif_list(parsed_args.node)
+
+        return (labels,
+                (oscutils.get_item_properties(s, columns) for s in data))
+
+
+class VifAttachBaremetalNode(command.Command):
+    """Attach VIF to a given node"""
+
+    log = logging.getLogger(__name__ + ".VifAttachBaremetalNode")
+
+    def get_parser(self, prog_name):
+        parser = super(VifAttachBaremetalNode, self).get_parser(prog_name)
+
+        parser.add_argument(
+            'node',
+            metavar='<node>',
+            help="Name or UUID of the node"
+        )
+        parser.add_argument(
+            'vif_id',
+            metavar='<vif-id>',
+            help="Name or UUID of the VIF to attach to a node."
+        )
+        parser.add_argument(
+            '--vif-info',
+            metavar='<key=value>',
+            action='append',
+            help="Record arbitrary key/value metadata. "
+                 "Can be specified multiple times. The mandatory 'id' "
+                 "parameter cannot be specified as a key.")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.log.debug("take_action(%s)", parsed_args)
+
+        baremetal_client = self.app.client_manager.baremetal
+
+        fields = utils.key_value_pairs_to_dict(parsed_args.vif_info or [])
+        baremetal_client.node.vif_attach(parsed_args.node, parsed_args.vif_id,
+                                         **fields)
+
+
+class VifDetachBaremetalNode(command.Command):
+    """Detach VIF from a given node"""
+
+    log = logging.getLogger(__name__ + ".VifDetachBaremetalNode")
+
+    def get_parser(self, prog_name):
+        parser = super(VifDetachBaremetalNode, self).get_parser(prog_name)
+
+        parser.add_argument(
+            'node',
+            metavar='<node>',
+            help="Name or UUID of the node"
+        )
+        parser.add_argument(
+            'vif_id',
+            metavar='<vif-id>',
+            help="Name or UUID of the VIF to detach from a node."
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        self.log.debug("take_action(%s)", parsed_args)
+
+        baremetal_client = self.app.client_manager.baremetal
+        baremetal_client.node.vif_detach(parsed_args.node, parsed_args.vif_id)
