@@ -376,15 +376,26 @@ class NodeShellTest(utils.BaseTestCase):
                           n_shell.do_node_set_maintenance,
                           client_mock, args)
 
-    def _do_node_set_power_state_helper(self, power_state):
+    def _do_node_set_power_state_helper(self, power_state,
+                                        soft=False, timeout=None, error=False):
         client_mock = mock.MagicMock()
         args = mock.MagicMock()
         args.node = 'node_uuid'
         args.power_state = power_state
+        args.soft = soft
+        args.power_timeout = timeout
 
-        n_shell.do_node_set_power_state(client_mock, args)
-        client_mock.node.set_power_state.assert_called_once_with('node_uuid',
-                                                                 power_state)
+        if error:
+            client_mock.node = mock.MagicMock()
+            client_mock.node.set_power_state = mock.MagicMock()
+            client_mock.node.set_power_state.side_effect = ValueError("fake")
+            self.assertRaises(exc.CommandError,
+                              n_shell.do_node_set_power_state,
+                              client_mock, args)
+        else:
+            n_shell.do_node_set_power_state(client_mock, args)
+            client_mock.node.set_power_state.assert_called_once_with(
+                'node_uuid', power_state, soft, timeout=timeout)
 
     def test_do_node_set_power_state_on(self):
         self._do_node_set_power_state_helper('on')
@@ -395,6 +406,37 @@ class NodeShellTest(utils.BaseTestCase):
     def test_do_node_set_power_state_reboot(self):
         self._do_node_set_power_state_helper('reboot')
 
+    def test_do_node_set_power_state_on_timeout(self):
+        self._do_node_set_power_state_helper('on', timeout=10)
+
+    def test_do_node_set_power_state_on_timeout_fail(self):
+        self._do_node_set_power_state_helper('on', timeout=0, error=True)
+
+    def test_do_node_set_power_state_off_timeout(self):
+        self._do_node_set_power_state_helper('off', timeout=10)
+
+    def test_do_node_set_power_state_reboot_timeout(self):
+        self._do_node_set_power_state_helper('reboot', timeout=10)
+
+    def test_do_node_set_power_state_soft_on_fail(self):
+        self._do_node_set_power_state_helper('on', soft=True, error=True)
+
+    def test_do_node_set_power_state_soft_off(self):
+        self._do_node_set_power_state_helper('off', soft=True)
+
+    def test_do_node_set_power_state_soft_reboot(self):
+        self._do_node_set_power_state_helper('reboot', soft=True)
+
+    def test_do_node_set_power_state_soft_on_timeout_fail(self):
+        self._do_node_set_power_state_helper('on', soft=True, timeout=10,
+                                             error=True)
+
+    def test_do_node_set_power_state_soft_off_timeout(self):
+        self._do_node_set_power_state_helper('off', soft=True, timeout=10)
+
+    def test_do_node_set_power_state_soft_reboot_timeout(self):
+        self._do_node_set_power_state_helper('reboot', soft=True, timeout=10)
+
     def test_do_node_set_target_raid_config_file(self):
         contents = '{"raid": "config"}'
 
@@ -1145,6 +1187,16 @@ class NodeShellTest(utils.BaseTestCase):
         client_mock.node.vif_attach.assert_called_once_with(
             'node_uuid', 'aaa-aaa')
 
+    def test_do_node_vif_attach_custom_fields(self):
+        client_mock = mock.MagicMock()
+        args = mock.MagicMock()
+        args.node = 'node_uuid'
+        args.vif_id = 'aaa-aaa'
+        args.vif_info = ['aaa=bbb', 'ccc=ddd']
+        n_shell.do_node_vif_attach(client_mock, args)
+        client_mock.node.vif_attach.assert_called_once_with(
+            'node_uuid', 'aaa-aaa', aaa='bbb', ccc='ddd')
+
     def test_do_node_vif_detach(self):
         client_mock = mock.MagicMock()
         args = mock.MagicMock()
